==> org-lsst-ccs-ui-2.4.4-sources.jar: org/lsst/ccs/gconsole/plugins/monitor/TabbedView.java <==
package org.lsst.ccs.gconsole.plugins.monitor;

import java.awt.BorderLayout;
import java.util.*;
import java.util.function.Function;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import org.lsst.ccs.bus.data.AgentInfo;
import org.lsst.ccs.gconsole.agent.AgentChannel;
import org.lsst.ccs.gconsole.agent.filter.PageFilter;
import org.lsst.ccs.gconsole.annotations.ConsoleLookup;
import org.lsst.ccs.subsystem.monitor.ui.tree.MonitorDisplay;

/**
 * Monitoring view that displays several other views in tabs.
 *
 * @author onoprien
 */
@ConsoleLookup(id="org.lsst.ccs.gconsole.plugins.monitor.MonitorView",
               name="Tabbed View",
               path="Built-In/Tabbed",
               description="Monitoring data view that displays its data in a set of tabs.")
public class TabbedView extends AbstractMonitorView1 {

// -- Fields : -----------------------------------------------------------------
    
    private final JComponent panel = new JPanel(new BorderLayout());
    private JTabbedPane tabbedPane;
    
    private final LinkedHashMap<String,AbstractMonitorView2> views = new LinkedHashMap<>();
    
    private Function<String,AbstractMonitorView2> viewFactory;
    
    

// -- Life cycle : -------------------------------------------------------------
    
    public TabbedView() {
        viewFactory = name -> new TreeView();
        panel.add(new MonitorDisplay.EMPTY());
    }
    
    
// -- Setters : ----------------------------------------------------------------
    
    /**
     * Sets the view to be displayed in the tab {@code name}.
     * @param name
     * @param view 
     */
    public void setTab(String name, AbstractMonitorView1 view) {
        // FIXME
    }
    
    /**
     * Sets the factory used to create views for tabs for which the view has not been explicitly set.
     * @param factory Factory that returns view instance given the tab name.
     */
    public void setViewFactory(Function<String,AbstractMonitorView2> factory) {
        viewFactory = factory;
    }
    
// -- Implementing MonitorView : -----------------------------------------------

    @Override
    public JComponent getPanel() {
        return panel;
    }


// -- Calling AbstractMonitorView2 hooks : --------------------------------------    

    @Override
    protected void connect(AgentInfo agent) {
        views.values().forEach(view -> view.connect(agent));
    }

    @Override
    protected void disconnect(AgentInfo agent) {
        views.values().forEach(view -> view.disconnect(agent));
    }

    @Override
    protected void addChannels(AgentInfo agent, Map<String, AgentChannel> channels) {
        Map<String,Map<String, AgentChannel>> allTabs = new LinkedHashMap<>();
        channels.forEach((path, ch) ->  {
            String[] ss = split(path);
            Map<String, AgentChannel> oneTab = allTabs.get(ss[0]);
            if (oneTab == null) {
                oneTab = new LinkedHashMap<>();
                allTabs.put(ss[0], oneTab);
            }
            oneTab.put(ss[1], ch);
        });
        allTabs.forEach((tab, cc) -> {
            AbstractMonitorView2 view = views.get(tab);
            if (view == null) {
                view = viewFactory.apply(tab);
                if (filter != null) {
                    view.setFilter(new PageFilter(filter, tab));
                }
                TreeMap<String,AbstractMonitorView2.Descriptor> m = getDescriptor().getTabs();
                if (m != null) {
                    AbstractMonitorView2.Descriptor d = m.get(tab);
                    if (d != null) {
                        view.restore(d);
                    }
                }
                views.put(tab, view);
                switch (views.size()) {
                    case 0:
                        tabbedPane = null;
                        panel.removeAll();
                        panel.add(new MonitorDisplay.EMPTY());
                        break;
                    case 1:
                        tabbedPane = null;
                        panel.removeAll();
                        panel.add(view.getPanel());
                        break;
                    case 2:
                        panel.removeAll();
                        tabbedPane = new JTabbedPane();
                        views.forEach((name,v) -> tabbedPane.add(name, v.getPanel()));
                        panel.add(tabbedPane);
                        break;
                    default:
                        tabbedPane.add(tab, view.getPanel());
                }
            }
            view.addChannels(agent, cc);
        });
    }

    @Override
    protected void removeChannels(AgentInfo agent, List<String> paths) {
        TreeMap<String,List<String>> allTabs = new TreeMap<>();
        paths.forEach(path ->  {
            String[] ss = split(path);
            List<String> oneTab = allTabs.get(ss[0]);
            if (oneTab == null) {
                oneTab = new ArrayList<>();
                allTabs.put(ss[0], oneTab);
            }
            oneTab.add(ss[1]);
        });
        allTabs.forEach((tab, cc) -> {
            AbstractMonitorView2 view = views.get(tab);
            if (view != null) {
                view.removeChannels(agent, cc);
                if (view.isEmpty()) {
                    views.remove(tab);
                    switch (views.size()) {
                        case 0:
                            tabbedPane = null;
                            panel.removeAll();
                            panel.add(new MonitorDisplay.EMPTY());
                            break;
                        case 1:
                            tabbedPane = null;
                            panel.removeAll();
                            panel.add(views.values().iterator().next().getPanel());
                            break;
                        default:
                            tabbedPane.remove(view.getPanel());
                    }
                }
            }
        });
    }

    @Override
    protected void updateChannels(AgentInfo agent, Map<String, Map.Entry<AgentChannel, List<String>>> channels) {
        TreeMap<String,Map<String, Map.Entry<AgentChannel, List<String>>>> allTabs = new TreeMap<>();
        channels.forEach((path, e) ->  {
            String[] ss = split(path);
            Map<String, Map.Entry<AgentChannel, List<String>>> oneTab = allTabs.get(ss[0]);
            if (oneTab == null) {
                oneTab = new HashMap<>();
                allTabs.put(ss[0], oneTab);
            }
            oneTab.put(ss[1], e);
        });
        allTabs.forEach((tab, cc) -> {
            AbstractMonitorView2 view = views.get(tab);
            if (view != null) {
                view.updateChannels(agent, cc);
            }
        });
    }

    
// -- Saving/restoring : -------------------------------------------------------
    
    /**
     * JavaBean that contains information required to re-create this view in its current state.
     */
    static public class Descriptor extends AbstractMonitorView2.Descriptor {

        private TreeMap<String, AbstractMonitorView2.Descriptor> tabs;

        public TreeMap<String, AbstractMonitorView2.Descriptor> getTabs() {
            return tabs;
        }

        public void setTabs(TreeMap<String, AbstractMonitorView2.Descriptor> tabs) {
            this.tabs = tabs;
        }
        
    }
    
    /**
     * Returns a descriptor that contains information required to re-create this view in its current state.
     * @return View descriptor.
     */
    @Override
    public Descriptor save() {
        Descriptor desc = getDescriptor();
        if (!views.isEmpty()) {
            TreeMap<String, AbstractMonitorView2.Descriptor> dd = desc.getTabs();
            if (dd == null) {
                dd = new TreeMap<>();
                desc.setTabs(dd);
            }
            for (Map.Entry<String,AbstractMonitorView2> e : views.entrySet()) {
                dd.put(e.getKey(), e.getValue().save());
            }
        }
        return desc;
    }
    
    /**
     * Restores this view to the state described by the provided descriptor, to the extent possible.
     * @param descriptor View descriptor.
     */
    @Override
    public void restore(MonitorView.Descriptor descriptor) {
        if (descriptor instanceof Descriptor) {
            Descriptor desc = (Descriptor) descriptor;
            this.descriptor = desc;
            TreeMap<String, AbstractMonitorView2.Descriptor> dd = desc.getTabs();
            if (dd != null) {
                for (Map.Entry<String, AbstractMonitorView2.Descriptor> e : dd.entrySet()) {
                    AbstractMonitorView2 v = views.get(e.getKey());
                    if (v != null) {
                        v.restore(e.getValue());
                    }
                }
            }
        }
    }

    @Override
    protected Descriptor getDescriptor() {
        if (descriptor == null) descriptor = new Descriptor();
        return (Descriptor) descriptor;
    }
    
    
// -- Local methods : ----------------------------------------------------------
    
    String[] split(String displayPath) {
        String[] out = new String[2];
        int i = displayPath.indexOf("//");
        if (i == -1) {
            out[0] = "";
            out[1] = displayPath;
        } else {
            out[0] = displayPath.substring(0, i);
            out[1] = displayPath.substring(i+2);
        }
        return out;
    }

}

==> org-lsst-ccs-ui-2.4.11-sources.jar: org/lsst/ccs/gconsole/plugins/monitor/TabbedView.java <==
package org.lsst.ccs.gconsole.plugins.monitor;

import java.awt.BorderLayout;
import java.util.*;
import java.util.function.Function;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import org.lsst.ccs.bus.data.AgentInfo;
import org.lsst.ccs.gconsole.agent.AgentChannel;
import org.lsst.ccs.gconsole.agent.filter.PageFilter;
import org.lsst.ccs.gconsole.annotations.ConsoleLookup;
import org.lsst.ccs.subsystem.monitor.ui.tree.MonitorDisplay;

/**
 * Monitoring view that displays several other views in tabs.
 *
 * @author onoprien
 */
@ConsoleLookup(id="org.lsst.ccs.gconsole.plugins.monitor.MonitorView",
               name="Tabbed View",
               path="Built-In/Tabbed",
               description="Monitoring data view that displays its data in a set of tabs.")
public class TabbedView extends AbstractMonitorView1 {

// -- Fields : -----------------------------------------------------------------
    
    private final JComponent panel = new JPanel(new BorderLayout());
    private JTabbedPane tabbedPane;
    
    private final LinkedHashMap<String,AbstractMonitorView2> views = new LinkedHashMap<>();
    
    private Function<String,AbstractMonitorView2> viewFactory;
    
    

// -- Life cycle : -------------------------------------------------------------
    
    public TabbedView() {
        viewFactory = name -> {
            AbstractMonitorView2 view = new TreeView();
            if (formatter != null) {
                view.setFormatter(formatter);
            }
            return view;
        };
        panel.add(new MonitorDisplay.EMPTY());
    }
    
    
// -- Setters : ----------------------------------------------------------------
    
    /**
     * Sets the view to be displayed in the tab {@code name}.
     * @param name
     * @param view 
     */
    public void setTab(String name, AbstractMonitorView1 view) {
        // FIXME
    }
    
    /**
     * Sets the factory used to create views for tabs for which the view has not been explicitly set.
     * @param factory Factory that returns view instance given the tab name.
     */
    public void setViewFactory(Function<String,AbstractMonitorView2> factory) {
        viewFactory = s -> {
            AbstractMonitorView2 view = factory.apply(s);
            if (formatter != null) {
                view.setFormatter(formatter);
            }
            return view;
        };
    }
    
// -- Implementing MonitorView : -----------------------------------------------

    @Override
    public JComponent getPanel() {
        return panel;
    }


// -- Calling AbstractMonitorView2 hooks : --------------------------------------    

    @Override
    protected void connect(AgentInfo agent) {
        views.values().forEach(view -> view.connect(agent));
    }

    @Override
    protected void disconnect(AgentInfo agent) {
        views.values().forEach(view -> view.disconnect(agent));
    }

    @Override
    protected void addChannels(AgentInfo agent, Map<String, AgentChannel> channels) {
        Map<String,Map<String, AgentChannel>> allTabs = new LinkedHashMap<>();
        channels.forEach((path, ch) ->  {
            String[] ss = split(path);
            Map<String, AgentChannel> oneTab = allTabs.get(ss[0]);
            if (oneTab == null) {
                oneTab = new LinkedHashMap<>();
                allTabs.put(ss[0], oneTab);
            }
            oneTab.put(ss[1], ch);
        });
        allTabs.forEach((tab, cc) -> {
            AbstractMonitorView2 view = views.get(tab);
            if (view == null) {
                view = viewFactory.apply(tab);
                if (filter != null) {
                    view.setFilter(new PageFilter(filter, tab));
                }
                TreeMap<String,AbstractMonitorView2.Descriptor> m = getDescriptor().getTabs();
                if (m != null) {
                    AbstractMonitorView2.Descriptor d = m.get(tab);
                    if (d != null) {
                        view.restore(d);
                    }
                }
                views.put(tab, view);
                switch (views.size()) {
                    case 0:
                        tabbedPane = null;
                        panel.removeAll();
                        panel.add(new MonitorDisplay.EMPTY());
                        break;
                    case 1:
                        tabbedPane = null;
                        panel.removeAll();
                        panel.add(view.getPanel());
                        break;
                    case 2:
                        panel.removeAll();
                        tabbedPane = new JTabbedPane();
                        views.forEach((name,v) -> tabbedPane.add(name, v.getPanel()));
                        panel.add(tabbedPane);
                        break;
                    default:
                        tabbedPane.add(tab, view.getPanel());
                }
            }
            view.addChannels(agent, cc);
        });
    }

    @Override
    protected void removeChannels(AgentInfo agent, List<String> paths) {
        TreeMap<String,List<String>> allTabs = new TreeMap<>();
        paths.forEach(path ->  {
            String[] ss = split(path);
            List<String> oneTab = allTabs.get(ss[0]);
            if (oneTab == null) {
                oneTab = new ArrayList<>();
                allTabs.put(ss[0], oneTab);
            }
            oneTab.add(ss[1]);
        });
        allTabs.forEach((tab, cc) -> {
            AbstractMonitorView2 view = views.get(tab);
            if (view != null) {
                view.removeChannels(agent, cc);
                if (view.isEmpty()) {
                    views.remove(tab);
                    switch (views.size()) {
                        case 0:
                            tabbedPane = null;
                            panel.removeAll();
                            panel.add(new MonitorDisplay.EMPTY());
                            break;
                        case 1:
                            tabbedPane = null;
                            panel.removeAll();
                            panel.add(views.values().iterator().next().getPanel());
                            break;
                        default:
                            tabbedPane.remove(view.getPanel());
                    }
                }
            }
        });
    }

    @Override
    protected void updateChannels(AgentInfo agent, Map<String, Map.Entry<AgentChannel, List<String>>> channels) {
        TreeMap<String,Map<String, Map.Entry<AgentChannel, List<String>>>> allTabs = new TreeMap<>();
        channels.forEach((path, e) ->  {
            String[] ss = split(path);
            Map<String, Map.Entry<AgentChannel, List<String>>> oneTab = allTabs.get(ss[0]);
            if (oneTab == null) {
                oneTab = new HashMap<>();
                allTabs.put(ss[0], oneTab);
            }
            oneTab.put(ss[1], e);
        });
        allTabs.forEach((tab, cc) -> {
            AbstractMonitorView2 view = views.get(tab);
            if (view != null) {
                view.updateChannels(agent, cc);
            }
        });
    }

    
// -- Saving/restoring : -------------------------------------------------------
    
    /**
     * JavaBean that contains information required to re-create this view in its current state.
     */
    static public class Descriptor extends AbstractMonitorView2.Descriptor {

        private TreeMap<String, AbstractMonitorView2.Descriptor> tabs;

        public TreeMap<String, AbstractMonitorView2.Descriptor> getTabs() {
            return tabs;
        }

        public void setTabs(TreeMap<String, AbstractMonitorView2.Descriptor> tabs) {
            this.tabs = tabs;
        }
        
    }
    
    /**
     * Returns a descriptor that contains information required to re-create this view in its current state.
     * @return View descriptor.
     */
    @Override
    public Descriptor save() {
        Descriptor desc = getDescriptor();
        if (!views.isEmpty()) {
            TreeMap<String, AbstractMonitorView2.Descriptor> dd = desc.getTabs();
            if (dd == null) {
                dd = new TreeMap<>();
                desc.setTabs(dd);
            }
            for (Map.Entry<String,AbstractMonitorView2> e : views.entrySet()) {
                dd.put(e.getKey(), e.getValue().save());
            }
        }
        return desc;
    }
    
    /**
     * Restores this view to the state described by the provided descriptor, to the extent possible.
     * @param descriptor View descriptor.
     */
    @Override
    public void restore(MonitorView.Descriptor descriptor) {
        if (descriptor instanceof Descriptor) {
            Descriptor desc = (Descriptor) descriptor;
            this.descriptor = desc;
            TreeMap<String, AbstractMonitorView2.Descriptor> dd = desc.getTabs();
            if (dd != null) {
                for (Map.Entry<String, AbstractMonitorView2.Descriptor> e : dd.entrySet()) {
                    AbstractMonitorView2 v = views.get(e.getKey());
                    if (v != null) {
                        v.restore(e.getValue());
                    }
                }
            }
        }
    }

    @Override
    protected Descriptor getDescriptor() {
        if (descriptor == null) descriptor = new Descriptor();
        return (Descriptor) descriptor;
    }
    
    
// -- Local methods : ----------------------------------------------------------
    
    String[] split(String displayPath) {
        String[] out = new String[2];
        int i = displayPath.indexOf("//");
        if (i == -1) {
            out[0] = "";
            out[1] = displayPath;
        } else {
            out[0] = displayPath.substring(0, i);
            out[1] = displayPath.substring(i+2);
        }
        return out;
    }

}
